@@ -13,11 +13,11 @@ from xdg import XDG_DATA_HOME, XDG_CONFIG_HOME
 
 from .error import (BasePathError, ProjectExistsError, ProjectDataMissingError,
         ProjectMissingError, TemplateDataMissingError, SystemDataMissingError,
-        GitExistsError, GitMissingError)
-from .term import link_echo
+        GitExistsError, GitMissingError, assert_never)
+from .term import link_echo, rm_echo
 
 if TYPE_CHECKING:
-    from typing import Optional, Tuple, Generator, List, Dict
+    from typing import Optional, Tuple, Generator, List, Dict, Union
 
 SHUTIL_ARCHIVE_FORMATS = [ar[0] for ar in shutil.get_archive_formats()]
 _suffix_map_helper = {
@@ -30,6 +30,20 @@ _suffix_map_helper = {
 SHUTIL_ARCHIVE_SUFFIX_MAP = {k:v for k, v in _suffix_map_helper.items()
         if v in SHUTIL_ARCHIVE_FORMATS}
 
+# todo: do something else with this
+def verbose_unlink(proj_info: ProjectInfo, path: Path):
+    if proj_info.verbose and path.exists():
+        rm_echo(path)
+    if not proj_info.dry_run:
+        try:
+            if path.is_file() or path.is_symlink():
+                path.unlink()
+            elif path.is_dir():
+                shutil.rmtree(path)
+        except Exception as err:
+            # todo: better error here
+            print(f"Failed to delete '{path}'. Reason: {err}")
+
 def _constant(func):
     """TODO: write"""
     def fset(self, value):
@@ -164,12 +178,12 @@ class _DataPath:
     @_constant
     def macro_dir(self) -> Path:
         """TODO: write"""
-        return self._resource_absolute / 'packages' / 'macros'
+        return self._resource_absolute / 'macros'
 
     @_constant
     def format_dir(self) -> Path:
         """TODO: write"""
-        return self._resource_absolute / 'packages' / 'format'
+        return self._resource_absolute / 'format'
 
     @_constant
     def citation_dir(self) -> Path:
@@ -384,6 +398,7 @@ class ProjectPath:
                 elif path.is_dir():
                     shutil.rmtree(path)
             except Exception as err:
+                # todo: better error here
                 print(f"Failed to delete '{path}'. Reason: {err}")
 
     def clear_temp(self) -> None:
@@ -396,6 +411,7 @@ class ProjectPath:
                     elif file_path.is_dir():
                         shutil.rmtree(file_path)
                 except Exception as err:
+                    # todo: better error here
                     print(f"Failed to delete '{file_path}'. Reason: {err}")
 
 class ProjectInfo(ProjectPath):
@@ -445,42 +461,65 @@ class _FileLinker(_BaseLinker):
         """TODO: write"""
         return f"{NAMES.prefix(self.name_convention)}{NAMES.prefix_separator}{name}"
 
+    def link_path(self, path: Path, rel_path: Path,
+            force:bool=False, silent_fail:bool=True, verbose=False, dry_run=False) -> bool:
+        source_path = path.resolve()
+        if source_path.suffix != self.suffix:
+            raise BasePathError(
+                    source_path,
+                    message=f"Filetype '{source_path.suffix}' is invalid!")
+        target_path = rel_path / (self.safe_name(source_path.name))
+
+        return self._link_helper(source_path, target_path, rel_path, str(path),
+            force=force, silent_fail=silent_fail, verbose=verbose, dry_run=dry_run)
+
     def link_name(self, name: str, rel_path: Path,
-            force:bool=False, silent_fail:bool=True, is_path:bool=False, verbose=False) -> None:
-        """TODO: write"""
-        if is_path:
-            source_path = Path(name).resolve()
-            if source_path.suffix != self.suffix:
-                raise BasePathError(
-                        source_path,
-                        message=f"Filetype '{source_path.suffix}' is invalid!")
-            target_path = rel_path / (self.safe_name(source_path.name))
-        else:
-            source_path = self.file_path(name).resolve()
-            target_path = rel_path / (self.safe_name(name) + self.suffix)
-            if not source_path.exists():
-                raise TemplateDataMissingError(
-                        source_path,
-                        user_str=self.user_str,
-                        name=name)
+            force:bool=False, silent_fail:bool=True, verbose=False, dry_run=False) -> bool:
+        """Return True if the link succeeds (either bc exists, or copied in), and False otherwise
+        """
+        source_path = self.file_path(name).resolve()
+        target_path = rel_path / (self.safe_name(name) + self.suffix)
+
+        return self._link_helper(source_path, target_path, rel_path, name,
+            force=force, silent_fail=silent_fail, verbose=verbose, dry_run=dry_run)
+
+    def _link_helper(self, source_path: Path, target_path: Path, rel_path: Path, echo_name: str,
+            force:bool=False, silent_fail:bool=True, verbose=False, dry_run=False) -> bool:
+
+        if not (source_path.exists() or target_path.exists()):
+            if verbose:
+                link_echo(self, echo_name, rel_path, mode='fail')
+            return False
+            # raise TemplateDataMissingError(
+            #         source_path,
+            #         user_str=self.user_str,
+            #         name=name)
 
         if target_path.exists() and not force:
+            if verbose:
+                link_echo(self, echo_name, rel_path, mode='exists')
             if silent_fail:
-                return
+                return True
             raise FileExistsError(
                     errno.EEXIST,
                     "Link target already exists",
                     str(target_path.resolve()))
 
+        # only print the message if the target path doesn't exist, or it if does and
+        # forced replace
+        # todo: same pattern as write_template (abstract out?)
         if verbose:
             if target_path.exists():
-                link_echo(self, name, rel_path, overwrite=True)
+                link_echo(self, echo_name, rel_path, mode='overwrite')
             else:
-                link_echo(self, name, rel_path, overwrite=False)
+                link_echo(self, echo_name, rel_path, mode='new')
 
-        shutil.copyfile(
-                str(source_path),
-                str(target_path.resolve()))
+        if not dry_run:
+            shutil.copyfile(
+                    str(source_path),
+                    str(target_path.resolve()))
+            return True
+        return False
 
 
 def _load_default_template() -> Dict:
