@@ -3,12 +3,13 @@ from __future__ import annotations
 import datetime
 import errno
 import os
+from pathlib import Path
 import stat
 from typing import TYPE_CHECKING
 
 from jinja2 import Environment, FileSystemLoader, TemplateNotFound
 
-from .error import SystemDataMissingError
+from .error import SystemDataMissingError, BasePathError
 from .filesystem import (DATA_PATH, JINJA_PATH,
         toml_dump, toml_load_local_template,
         macro_linker, format_linker, citation_linker, template_linker)
@@ -16,7 +17,7 @@ from .term import render_echo, init_echo
 
 if TYPE_CHECKING:
     from pathlib import Path
-    from typing import Optional, Iterable, Dict
+    from typing import Optional, Iterable, Dict, List
     from jinja2 import Template
     from .filesystem import Config, ProjectInfo, _FileLinker
 
@@ -65,7 +66,7 @@ class GenericTemplate:
             date=datetime.date.today())
 
     def write_template(self, template_path: Path, target_path: Path, config: Config,
-            force:bool=False, verbose=False) -> None:
+            force:bool=False, verbose=False, dry_run=False) -> None:
         """Write template at location template_path to file at location
         target_path. Overwrite target_path when force=True."""
         if target_path.exists() and not force:
@@ -73,18 +74,19 @@ class GenericTemplate:
                     errno.EEXIST,
                     "Template write location aready exists",
                     str(target_path.resolve()))
+        # todo: same pattern as _link_helper (abstract out?)
+        if verbose:
+            if target_path.exists():
+                render_echo(template_path, target_path, overwrite=True)
+            else:
+                render_echo(template_path, target_path, overwrite=False)
         try:
-            if verbose:
-                if target_path.exists():
-                    render_echo(template_path, target_path, overwrite=True)
-                else:
-                    render_echo(template_path, target_path, overwrite=False)
-
-            target_path.parent.mkdir(parents=True, exist_ok=True)
-            target_path.write_text(
-                self.render_template(
-                    self.env.get_template(str(template_path)),
-                    config))
+            if not dry_run:
+                target_path.parent.mkdir(parents=True, exist_ok=True)
+                target_path.write_text(
+                    self.render_template(
+                        self.env.get_template(str(template_path)),
+                        config))
 
         except TemplateNotFound as err:
             raise SystemDataMissingError(template_path) from err
@@ -101,18 +103,34 @@ class ProjectTemplate(GenericTemplate):
             template_path: Path, target_path: Path,
             force:bool=False, executable:bool=False) -> None:
         """TODO: write"""
-        if not proj_info.dry_run:
-            self.write_template(
-                    template_path,
-                    target_path,
-                    proj_info.config,
-                    force=force,
-                    verbose=proj_info.verbose)
-            if executable:
-                os.chmod(target_path, stat.S_IXUSR |  stat.S_IWUSR | stat.S_IRUSR)
+        self.write_template(
+                template_path,
+                target_path,
+                proj_info.config,
+                force=force,
+                verbose=proj_info.verbose,
+                dry_run=proj_info.dry_run)
+        if not proj_info.dry_run and executable:
+            os.chmod(target_path, stat.S_IXUSR |  stat.S_IWUSR | stat.S_IRUSR)
 
     def write_tpr_files(self, proj_info: ProjectInfo) -> None:
         """Create texproject project data directory and write files."""
+        linker = PackageLinker(proj_info, force=False, silent_fail=True)
+
+        failed = {
+                'macros': linker.link_macros(self.template_dict['macros']),
+                'citations': linker.link_citations(self.template_dict['citations'])
+                }
+
+        # missing format file is really bad
+        if linker.link_format(self.template_dict['format']):
+            # todo: make this better
+            raise Exception("Missing format file!")
+
+        # update the template
+        for k,v in failed.items():
+            self.template_dict[k] = [it for it in self.template_dict[k] if it not in v]
+
         self.write_template_with_info(proj_info,
                 JINJA_PATH.classinfo,
                 proj_info.classinfo,
@@ -122,10 +140,8 @@ class ProjectTemplate(GenericTemplate):
                 proj_info.bibinfo,
                 force=True)
 
-        linker = PackageLinker(proj_info, force=False, silent_fail=True)
-        linker.link_macros(self.template_dict['macros'])
-        linker.link_citations(self.template_dict['citations'])
-        linker.link_format(self.template_dict['format'])
+        # todo: if something failed, still must raise an exception here
+        # that way, the return code is correct
 
     def write_git_files(self, proj_info: ProjectInfo, force: bool=False) -> None:
         """TODO: write"""
@@ -151,10 +167,9 @@ class ProjectTemplate(GenericTemplate):
 
 class InitTemplate(ProjectTemplate):
     """TODO: write"""
-    def __init__(self, template_name: str, citations: Iterable[str]):
+    def __init__(self, template_name: str):
         """Load the template generator from a template name."""
         template_dict = template_linker.load_template(template_name)
-        template_dict['citations'].extend(citations)
         self.template_name = template_name
         super().__init__(template_dict)
 
@@ -190,35 +205,50 @@ class LoadTemplate(ProjectTemplate):
 class PackageLinker:
     """TODO: write"""
     def __init__(self, proj_info: ProjectInfo,
-            force:bool=False, silent_fail:bool=True,
-            is_path:bool=False) -> None:
+            force:bool=False, silent_fail:bool=True) -> None:
         """TODO: write"""
         self.proj_info = proj_info
         self.force = force
         self.silent_fail = silent_fail
-        self.is_path = is_path
 
-    def make_link(self, linker: _FileLinker, name: str) -> None:
-        """Helper function for linking."""
-        if not self.proj_info.dry_run:
-            linker.link_name(name,
-                    self.proj_info.data_dir,
-                    is_path=self.is_path,
-                    force=self.force,
-                    silent_fail=self.silent_fail,
-                    verbose=self.proj_info.verbose)
+    def make_link(self, linker: _FileLinker, name: str) -> bool:
+        """Helper function for linking. Returns True if the link was created, False if it failed,
+        and None if no attempt was made"""
+        return linker.link_name(name,
+                self.proj_info.data_dir,
+                force=self.force,
+                silent_fail=self.silent_fail,
+                verbose=self.proj_info.verbose,
+                dry_run = self.proj_info.dry_run)
 
-    def link_macros(self, macro_list: Iterable[str]) -> None:
+    def make_path_link(self, linker: _FileLinker, name: Path) -> bool:
+        """Helper function for linking. Returns True if the link was created, False if it failed,
+        and None if no attempt was made"""
+        return linker.link_path(name,
+                self.proj_info.data_dir,
+                force=self.force,
+                silent_fail=self.silent_fail,
+                verbose=self.proj_info.verbose,
+                dry_run = self.proj_info.dry_run)
+
+    def link_macros(self, macro_list: Iterable[str]) -> List[str]:
+        """Returns a list of macros where the inclusion failed."""
+        return [macro for macro in macro_list if not self.make_link(macro_linker, macro)]
+
+    def link_macro_paths(self, macro_list: Iterable[Path]) -> List[bool]:
         """TODO: write"""
-        for macro in macro_list:
-            self.make_link(macro_linker, macro)
+        return [self.make_path_link(macro_linker, macro) for macro in macro_list]
 
-    def link_citations(self, citation_list: Iterable[str]) -> None:
+    def link_citations(self, citation_list: Iterable[str]) -> List[str]:
+        """Returns a list of citations where the inclusion failed."""
+        return [citation for citation in citation_list if not self.make_link(citation_linker, citation)]
+
+    def link_citation_paths(self, citation_list: Iterable[Path]) -> List[bool]:
         """TODO: write"""
-        for cit in citation_list:
-            self.make_link(citation_linker, cit)
+        return [self.make_path_link(citation_linker, citation) for citation in citation_list]
 
-    def link_format(self, fmt: Optional[str]) -> None:
+    def link_format(self, fmt: Optional[str]) -> Optional[str]:
         """TODO: write"""
         if fmt is not None:
-            self.make_link(format_linker, fmt)
+            if not self.make_link(format_linker, fmt):
+                return fmt
